# -*- coding: utf-8 -*-

import torch
import torch.nn as nn


class BasicCritic(nn.Module):
    """
    The BasicCritic module takes an image and predicts whether it is a cover
    image or a steganographic image (N, 1).

    Input: (N, 3, H, W)
    Output: (N, 1)
    """

    def __init__(self, hidden_size):

        super(BasicCritic, self).__init__()

        self.layers = nn.Sequential(
            nn.Conv2d(in_channels=3, out_channels=hidden_size, kernel_size=3),
            nn.LeakyReLU(inplace=True),
            nn.BatchNorm2d(hidden_size),

            nn.Conv2d(in_channels=hidden_size, out_channels=hidden_size, kernel_size=3),
            nn.LeakyReLU(inplace=True),
            nn.BatchNorm2d(hidden_size),

            nn.Conv2d(in_channels=hidden_size, out_channels=hidden_size, kernel_size=3),
            nn.LeakyReLU(inplace=True),
            nn.BatchNorm2d(hidden_size),

            nn.Conv2d(in_channels=hidden_size, out_channels=1, kernel_size=3)
        )

    def forward(self, x):

        x = self.layers(x)
        x = torch.mean(x.view(x.size(0), -1), dim=1)

        return x



# -*- coding: utf-8 -*-

import torch


def assert_called_with_tensors(mock, calls):
    assert mock.call_count == len(calls), "Different number of calls"

    for mock_call, real_call in zip(mock.call_args_list, calls):
        assert len(mock_call) == len(real_call) - 1, "Different number of arguments"

        for mock_arg, arg in zip(mock_call[0], real_call[1]):
            if isinstance(mock_arg, torch.Tensor):
                assert (mock_arg == arg).all()
            else:
                assert mock_arg == arg



# -*- coding: utf-8 -*-
"""Top-level package for SteganoGAN."""

__author__ = """MIT Data To AI Lab"""
__email__ = 'dailabmit@gmail.com'
__version__ = '0.1.0'

from steganogan import cli
from steganogan.models import SteganoGAN

__all__ = ('SteganoGAN', 'cli')



# -*- coding: utf-8 -*-

import numpy as np
import torch
import torchvision
from torchvision import transforms

_DEFAULT_MU = [.5, .5, .5]
_DEFAULT_SIGMA = [.5, .5, .5]

DEFAULT_TRANSFORM = transforms.Compose([
    transforms.RandomHorizontalFlip(),
    transforms.RandomCrop(360, pad_if_needed=True),
    transforms.ToTensor(),
    transforms.Normalize(_DEFAULT_MU, _DEFAULT_SIGMA),
])


class ImageFolder(torchvision.datasets.ImageFolder):
    def __init__(self, path, transform, limit=np.inf):
        super().__init__(path, transform=transform)
        self.limit = limit

    def __len__(self):
        length = super().__len__()
        return min(length, self.limit)


class DataLoader(torch.utils.data.DataLoader):

    def __init__(self, path, transform=None, limit=np.inf, shuffle=True,
                 num_workers=4, batch_size=4, *args, **kwargs):

        if transform is None:
            transform = DEFAULT_TRANSFORM

        super().__init__(
            ImageFolder(path, transform, limit),
            batch_size=batch_size,
            shuffle=shuffle,
            num_workers=num_workers,
            *args,
            **kwargs
        )
